<?php

namespace Drp\NativeEventApp\Test\Api;

use Carbon\Carbon;
use Drp\NativeEventApp\Models\Event;
use Drp\NativeEventApp\Models\Page;
use Drp\NativeEventApp\Models\Registration;
use Drp\NativeEventApp\Models\Segment;
use Drp\NativeEventApp\Modules\Modules;
use Drp\NativeEventApp\Modules\Types\SimpleContainer;
use Drp\NativeEventApp\Modules\Types\Text;
use Drp\NativeEventApp\Templates\DataProviders\DataProvider;
use Drp\NativeEventApp\Templates\DataProviders\DataProviderRegistry;
use Spatie\Snapshots\MatchesSnapshots;

class PageTest extends GraphQLTestCase
{
    use MatchesSnapshots;

    /** @test */
    public function returns_modules_with_parsed_content(): void
    {
        Carbon::setTestNow('01-01-2019');
        [$user, $event] = $this->registerFakeEvent();

        Registration::create([
            'authId' => $user->getAuthIdentifier(),
            'eventId' => $event->id,
            'forename' => 'John',
            'surname' => 'Smith',
        ]);

        app(DataProviderRegistry::class)->bind('user', function (Registration $registration) {
            $data = ['forename' => $registration->forename, 'lastName' => $registration->surname];
            return new class($data) extends DataProvider {
                /** @var string */
                public $forename;

                /** @var string */
                public $lastName;
            };
        });

        factory(Page::class)->create([
            'eventId' => $event->id,
            'modules' => Modules::make([
                Text::make('hello {user.forename} {user.lastName}'),
                Text::make('Hi there {user.lastName}'),
            ]),
        ]);

        $query = '{
            events(id:1){
                id,
                pages {
                    modules
                }
            }
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function must_match_segments(): void
    {
        [$user, $event] = $this->registerFakeEvent();
        [$segmentA, $segmentB] = factory(Segment::class)->times(2)->create();

        $registration = Registration::create([
            'authId' => $user->getAuthIdentifier(),
            'eventId' => $event->id,
        ]);
        $registration->joinSegments(collect([$segmentA]));

        $pageA = factory(Page::class)->create(['eventId' => $event->id]);
        $pageA->joinSegments(collect([$segmentA]));

        $pageB = factory(Page::class)->create(['eventId' => $event->id]);
        $pageB->joinSegments(collect([$segmentA, $segmentB]));

        $pageC = factory(Page::class)->create(['eventId' => $event->id]);

        $pageD = factory(Page::class)->create(['eventId' => $event->id]);
        $pageD->joinSegments(collect([$segmentB]));

        $query = '{
            events(id:1){
                id,
                pages {
                    id
                }
            }
        }';

        $response = $this->graphQlRequest($query);

        $this->assertMatchesJsonSnapshot($response);
    }

    /** @test */
    public function must_match_multiple_segments(): void
    {
        [$user, $event] = $this->registerFakeEvent();
        [$segmentA, $segmentB, $segmentC] = factory(Segment::class)->times(3)->create();

        $registration = Registration::create([
            'authId' => $user->getAuthIdentifier(),
            'eventId' => $event->id,
        ]);
        $registration->joinSegments(collect([$segmentA, $segmentB]));

        $pageA = factory(Page::class)->create(['eventId' => $event->id]);
        $pageA->joinSegments(collect([$segmentA]));

        $pageB = factory(Page::class)->create(['eventId' => $event->id]);
        $pageB->joinSegments(collect([$segmentA, $segmentB]));

        $pageC = factory(Page::class)->create(['eventId' => $event->id]);
        $pageC->joinSegments(collect([$segmentB, $segmentC]));

        $pageD = factory(Page::class)->create(['eventId' => $event->id]);
        $pageD->joinSegments(collect([$segmentA, $segmentB, $segmentC]));

        $query = '{
            events(id:1){
                id,
                pages {
                    id
                }
            }
        }';

        $response = $this->graphQlRequest($query);

        $this->assertMatchesJsonSnapshot($response);
    }

    /** @test */
    public function modules_can_be_scoped_by_segment(): void
    {
        [$user, $event] = $this->registerFakeEvent();
        [$segmentA, $segmentB] = factory(Segment::class)->times(2)->create();

        $registration = Registration::create([
            'authId' => $user->getAuthIdentifier(),
            'eventId' => $event->id,
        ]);
        $registration->joinSegments(collect([$segmentA]));

        factory(Page::class)->create([
            'eventId' => $event->id,
            'modules' => Modules::make([
                Text::make('EVERYONE SHOULD SEE THIS'),
                Text::make('SCOPE TO SEGMENT A')->viewableBy(collect([$segmentA])),
                Text::make('SHOULD NOT SEE THIS MODULE')->viewableBy(collect([$segmentB])),
                SimpleContainer::make()->children([
                    Text::make('SCOPE TO SEGMENT A')->viewableBy(collect([$segmentA])),
                    Text::make('SHOULD NOT SEE THIS MODULE')->viewableBy(collect([$segmentB])),
                ]),
            ]),
        ]);

        $query = '{
            events(id:1){
                id,
                pages {
                    modules
                }
            }
        }';

        $response = $this->graphQlRequest($query);

        $this->assertMatchesJsonSnapshot($response);
    }

    /** @test */
    public function will_ignore_pages_that_authenticatable_does_not_have_access_to(): void
    {
        [$user, $event] = $this->registerFakeEvent();

        Registration::create([
            'authId' => $user->getAuthIdentifier(),
            'eventId' => $event->id,
        ]);

        $segment = factory(Segment::class)->create();
        $page = factory(Page::class)->create([
            'eventId' => $event->id,
        ]);
        $page->joinSegments(collect([$segment]));

        $query = '{
            events(id:1){
                id,
                pages {
                    title
                }
            }
        }';

        $response = $this->graphQlRequest($query);

        $this->assertMatchesJsonSnapshot($response);
    }

    /** @test */
    public function will_ignore_segments_in_modules_that_dont_exist(): void
    {
        [$user, $event] = $this->registerFakeEvent();

        Registration::create([
            'authId' => $user->getAuthIdentifier(),
            'eventId' => $event->id,
        ]);

        factory(Page::class)->create([
            'eventId' => $event->id,
            'modules' => Modules::make([
                Text::make('SHOULD SEE THIS')->viewableBy(collect([(object) ['id' => 1]])),
            ]),
        ]);

        $query = '{
            events(id:1){
                id,
                pages {
                    modules
                }
            }
        }';

        $response = $this->graphQlRequest($query);

        $this->assertMatchesJsonSnapshot($response);
    }

    /** @test */
    public function modules_can_have_styles(): void
    {
        Carbon::setTestNow('01-01-2019');
        [$user, $event] = $this->registerFakeEvent();

        Registration::create([
            'authId' => $user->getAuthIdentifier(),
            'eventId' => $event->id,
        ]);

        factory(Page::class)->create([
            'eventId' => $event->id,
            'modules' => Modules::make([
                Text::make('Hello World')
                    ->styles(['container' => ['padding' => 16]])
                    ->className('narrow'),
            ]),
        ]);

        $query = '{
            events(id:1){
                id,
                pages {
                    modules
                }
            }
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function get_all_pages()
    {
        Carbon::setTestNow('01-01-2019');
        [$user, $event] = $this->registerFakeEvent();

        Registration::create([
            'authId' => $user->getAuthIdentifier(),
            'eventId' => $event->id,
        ]);

        factory(Page::class)->times(2)->create([
            'eventId' => $event->id,
        ]);

        $query = '{
            events(id:1){
                id,
                pages {
                    title
                }
            }
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function gets_page_by_id()
    {
        Carbon::setTestNow('01-01-2019');
        [$user, $event] = $this->registerFakeEvent();

        Registration::create([
            'authId' => $user->getAuthIdentifier(),
            'eventId' => $event->id,
        ]);

        factory(Page::class)->create([
            'eventId' => $event->id,
            'title' => 'Should receive',
        ]);

        factory(Page::class)->create([
            'eventId' => $event->id,
            'title' => 'Should not receive',
        ]);

        $query = '{
            events(id:1){
                id,
                pages(id:1) {
                    title
                }
            }
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }
}
